"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const performance_now_1 = __importDefault(require("performance-now"));
const common_1 = require("./common");
function createAutometricPromise(name, baseOptions = {}) {
    const register = common_1.createRegistry();
    const { registers = [] } = baseOptions;
    registers.push(register);
    const metrics = {
        calls: common_1.createCounter({
            help: `Autometric Counter for calls of the "${name}" Promise`,
            name: name + "_calls_total",
            registers,
        }),
        durations: common_1.createSummary({
            help: `Autometric Summary for the durations of the "${name}" Promise`,
            name: name + "_durations_ms",
            registers,
        }),
    };
    baseOptions.promise = baseOptions.promise || Promise;
    return _a = class AutometricPromise extends baseOptions.promise {
            constructor(executor, options = {}) {
                const startTimestamp = performance_now_1.default();
                const startDate = new Date();
                options.labels = options.labels || {};
                options.rewriteLabels = options.rewriteLabels || ((passLabels) => passLabels);
                let currentLabels = Object.assign({}, AutometricPromise.labels, options.labels);
                metrics.calls.inc(currentLabels, 1, startDate);
                super((resolve, reject) => {
                    const intResolve = (result) => {
                        const endTimestamp = performance_now_1.default();
                        if (!options.noStatusLabel) {
                            currentLabels = Object.assign({}, currentLabels, { status: "resolve" });
                        }
                        currentLabels = options.rewriteLabels(currentLabels, result, baseOptions, options);
                        metrics.durations.observe(currentLabels, endTimestamp - startTimestamp);
                        resolve(result);
                    };
                    const intReject = (reason) => {
                        const endTimestamp = performance_now_1.default();
                        if (!options.noStatusLabel) {
                            currentLabels = Object.assign({}, currentLabels, { status: "reject" });
                        }
                        currentLabels = options.rewriteLabels(currentLabels, reason, baseOptions, options);
                        metrics.durations.observe(currentLabels, endTimestamp - startTimestamp);
                        reject(reason);
                    };
                    executor(intResolve, intReject);
                });
            }
        },
        _a.register = register,
        _a.prefix = name,
        _a.labels = baseOptions.labels || {},
        _a.all = Promise.all,
        _a.race = Promise.race,
        _a.reject = Promise.reject,
        _a.resolve = Promise.resolve,
        _a;
    var _a;
}
exports.createAutometricPromise = createAutometricPromise;
