"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const prom_client_1 = require("prom-client");
const _1 = require("./");
const index_1 = require("./index");
describe("AutometricPromise", () => {
    describe("creation of the Promise class", () => {
        it("should return an instance of a Promise", () => {
            chai_1.expect(new (_1.createAutometricPromise("instance_of_promise"))((resolve) => resolve(true))).instanceOf(Promise);
        });
        describe("static properties", () => {
            let AutometricPromise;
            const name = "static_properties";
            const labels = { label: "just-a-test" };
            class ExtendedPromsise extends Promise {
                constructor() {
                    super(...arguments);
                    this.extendedPromise = true;
                }
            }
            before(() => {
                index_1.autometricRegister.getSingleMetric("autometric_counters_total").reset();
                index_1.autometricRegister.getSingleMetric("autometric_registers_total").reset();
                index_1.autometricRegister.getSingleMetric("autometric_summaries_total").reset();
            });
            before(() => {
                AutometricPromise = _1.createAutometricPromise(name, {
                    labels,
                    promise: ExtendedPromsise,
                });
            });
            it("should be an instance of the extended Promise", () => {
                chai_1.expect(new AutometricPromise((resolve) => resolve(true))).instanceOf(Promise);
            });
            it("should have the name as prefix attribute", () => {
                chai_1.expect(AutometricPromise.prefix).to.equal(name);
            });
            it("should have the labels as labels attribute", () => {
                chai_1.expect(AutometricPromise.labels).to.deep.equal(labels);
            });
            it("should have the calls counter in the register", () => {
                chai_1.expect(AutometricPromise.register.getSingleMetric(name + "_calls_total")).instanceOf(prom_client_1.Counter);
            });
            it("should have the durations summary in the register", () => {
                chai_1.expect(AutometricPromise.register.getSingleMetric(name + "_durations_ms")).instanceOf(prom_client_1.Summary);
            });
            it("should count the creation of a counters", () => void chai_1.expect(index_1.autometricRegister.getSingleMetric("autometric_counters_total").get().values[0].value).equal(1));
            it("should count the creation of a register", () => void chai_1.expect(index_1.autometricRegister.getSingleMetric("autometric_registers_total").get().values[0].value).equal(1));
            it("should count the creation of a summaries", () => void chai_1.expect(index_1.autometricRegister.getSingleMetric("autometric_summaries_total").get().values[0].value).equal(1));
            it("should have a static all function", () => __awaiter(this, void 0, void 0, function* () {
                const result = yield AutometricPromise.all([Promise.resolve(true), AutometricPromise.resolve(true)]);
                chai_1.expect(result).deep.equal([true, true]);
            }));
            it("should have a static race function", () => __awaiter(this, void 0, void 0, function* () {
                const result = yield AutometricPromise.race([
                    Promise.resolve(true),
                    new Promise((resolve) => { setTimeout(() => { resolve(false); }, 100); }),
                ]);
                chai_1.expect(result).deep.equal(true);
            }));
            it("should have a static resolve function", () => __awaiter(this, void 0, void 0, function* () {
                const result = yield AutometricPromise.resolve(true);
                chai_1.expect(result).deep.equal(true);
            }));
            it("should have a static reject function", () => {
                return AutometricPromise
                    .reject(new Error("test"))
                    .then(() => {
                    /* istanbul ignore next */
                    return Promise.reject(new Error("Error not caught"));
                })
                    .catch((err) => {
                    chai_1.expect(err.message).equal("test");
                });
            });
        });
    });
    describe("counters", () => {
        let AutometricPromise;
        const name = "counters";
        before(() => {
            AutometricPromise = _1.createAutometricPromise(name);
        });
        it("should count a call", () => {
            /* tslint:disable:no-unused-expression */
            new AutometricPromise(() => {
                // do nothing here...
            });
            chai_1.expect(AutometricPromise.register.getSingleMetric(name + "_calls_total").get().values[0].value).equal(1);
        });
        it("should count rejects", (done) => {
            new AutometricPromise((resolve, reject) => {
                reject(new Error("Expected behavior"));
            }).catch(() => {
                chai_1.expect(AutometricPromise.register.getSingleMetric(name + "_durations_ms")
                    .get().values
                    .reduce((prev, elem) => {
                    if (elem.metricName === name + "_durations_ms_count") {
                        return elem.value;
                    }
                    return prev;
                }, null)).equal(1);
                chai_1.expect(AutometricPromise.register.getSingleMetric(name + "_durations_ms")
                    .get().values
                    .reduce((prev, elem) => {
                    if (elem.metricName === name + "_durations_ms_count") {
                        return elem.labels;
                    }
                    return prev;
                }, null)).deep.equal({ status: "reject" });
                done();
            });
        });
        it("should count resolves", (done) => {
            AutometricPromise.register.getSingleMetric(name + "_durations_ms").reset();
            (new AutometricPromise((resolve) => {
                resolve(true);
            })).then(() => {
                chai_1.expect(AutometricPromise.register.getSingleMetric(name + "_durations_ms")
                    .get().values
                    .reduce((prev, elem) => {
                    if (elem.metricName === name + "_durations_ms_count") {
                        return elem.value;
                    }
                    return prev;
                }, null)).equal(1);
                chai_1.expect(AutometricPromise.register.getSingleMetric(name + "_durations_ms")
                    .get().values
                    .reduce((prev, elem) => {
                    if (elem.metricName === name + "_durations_ms_count") {
                        return elem.labels;
                    }
                    return prev;
                }, null)).deep.equal({ status: "resolve" });
                done();
            });
        });
    });
    describe("summary", () => {
        let AutometricPromise;
        const name = "summaries";
        before(() => {
            AutometricPromise = _1.createAutometricPromise(name);
        });
        it("should measure execution duration", () => {
            return new AutometricPromise((resolve) => {
                setTimeout(() => {
                    resolve(true);
                }, 100);
            }).then(() => {
                const values = AutometricPromise.register.getSingleMetric(name + "_durations_ms")
                    .get().values;
                chai_1.expect(values[values.length - 2].value).greaterThan(99).lessThan(110);
            });
        });
    });
});
